'use strict';

Object.defineProperty(exports, '__esModule', {
    value: true
});

exports['default'] = function (_ref) {
    var Plugin = _ref.Plugin;
    var t = _ref.types;

    /**
     * Plugin Name
     * @type {String}
     */
    var pluginName = 'babel-plugin-template-string-processing';

    /**
     * Processes string with given processors
     * @param {String} string - code to be evaluated
     * @param {Array} options - an array of processors to be applied
     * @return {String} processed code
     */
    var processor = function processor(string, options) {
        options.forEach(function (transformer) {
            var module = require('template-string-processing-' + transformer.type);
            string = module(string, transformer);
        });
        return string;
    };

    /**
     * Selects appropriate processors based on the beginning of the string
     * @param {Array} quasis - node quasis, array of objects
     * @param {Array} options - array of all known processors
     * @return {Array} array of processors that should be applied to the string
     */
    var selectProcessor = function selectProcessor(quasis, options) {
        var stringStart = quasis.length > 0 ? String(quasis[0].value.cooked) : '';

        return options.filter(function (option) {
            return stringStart.startsWith('~' + option.type);
        });
    };

    return new Plugin(pluginName, {
        visitor: {
            TemplateLiteral: function TemplateLiteral(node, parent, scope, file) {
                var options = file.opts.extra.hasOwnProperty(pluginName) ? file.opts.extra[pluginName] : null;
                var processors = selectProcessor(node.quasis, options);

                if (processors.length > 0) {
                    var strings = [];

                    var _iteratorNormalCompletion = true;
                    var _didIteratorError = false;
                    var _iteratorError = undefined;

                    try {
                        for (var _iterator = node.quasis[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                            var elem = _step.value;

                            strings.push(elem.value.cooked);
                        }

                        // @todo: find better way of processing whole string
                    } catch (err) {
                        _didIteratorError = true;
                        _iteratorError = err;
                    } finally {
                        try {
                            if (!_iteratorNormalCompletion && _iterator['return']) {
                                _iterator['return']();
                            }
                        } finally {
                            if (_didIteratorError) {
                                throw _iteratorError;
                            }
                        }
                    }

                    strings = processor(strings.join('${###}'), options).split('${###}');

                    for (var elem in node.quasis) {
                        node.quasis[elem].value.cooked = strings[elem];
                    }

                    return node;
                }
            }
        }
    });
};

;
module.exports = exports['default'];